#!/usr/bin/env python
# -*- coding: utf-8 -*-
########################################################################
#
# wpsh/commands.py
# $Id: commands.py,v 1.3 2009/09/20 02:31:37 elias Alpha1 $
#
# The WordPress Shell
# Copyright (c) 2009 Elias Schwerdtfeger, http://www.tamagothi.de/ 
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
#
########################################################################


"""\
Commands

For every command a little function is given, which implements it.

In case of an exception, the function pointed by exchandler is called,
which may be overwritten. 

The verbose messages are given by the function pointed by vmsg, which
may be overwritten. Your own implementation should respect the verbose
variable and be silent if no verbose output is requested. Do not use
any kind of output which blocks the user, no dialogs or similar stuff.

None of the functions is this module have any return value. They just
implement commands, sometimes a little interactively.

"""


ID = '$Id: commands.py,v 1.3 2009/09/20 02:31:37 elias Alpha1 $'



import sys
import locale
import re
import codecs

import blog
import wpshell
import wpshbase
import wptypes as wt



verbose = False
theConfig = blog.theConfig



class BlognameAlreadyExists(wpshbase.Error):
    """Avoid double blognames"""
    pass



class MalformedIdentifer(wpshbase.Error):
    """Some text is malformed"""
    pass



class CategoryError(wpshbase.Error):
    """A general category error"""
    pass



class CategoryDoesNotExist(CategoryError):
    """A cat does not exist"""
    pass



class CategoryAlreadyExist(CategoryError):
    """A category already exists"""
    pass



def __exception(msg, *args):
    """\
    Handle an exception.
    """
    rmsg = msg % args
    (typ, val, tb) = sys.exc_info()
    doc = typ.__doc__
    sys.stderr.write('%s - %s\n' % (doc, val))
    sys.stderr.write('%s\n' % (rmsg,))
    if wpshbase.DEBUG:
        # This isn't the wanted kind of output for a normal user!
        import traceback
        traceback.print_tb(tb)



def __vmsg(fmt, *args):
    """\
    Print a verbose message.
    """
    if verbose:
        print ' * [', fmt % args, ']'



def __attmsg(fmt, *args):
    """\
    Print an important message.
    """
    print "ATTENTION:", fmt % args



def __done():
    """\
    Say that everything is okay in verbose mode.
    """
    vmsg('Done.')



def __uinput(prompt, *args):
    """\
    A local input function.
    """
    # Arrgh! There are encodings problems! How I hate python for its
    # sucking string handling! I want to use umlauts, and speakers of
    # other languages probably want to use there funny diacritics and
    # characters. By centralizing all input I hope to fix it in a
    # single place...
    p = prompt % args
    d = raw_input(p)
    return d



# You may want to change these defaults, if you do something magic
# based on this code (e.g. a GUI)...

exchandler = __exception
vmsg = __vmsg
done = __done
attmsg = __attmsg
uinput = __uinput



def listBlogs():
    try:
        tf = wpshbase.TupleFormatter()
        vmsg('List of configured blogs')
        blst = theConfig.getBlogList()
        if len(blst) == 0:
            print 'There are no blogs configured yet'
        else:
            for i in blst:
                bc = theConfig.getBlogConfig(i)
                tf.add(bc.name, bc.comment)
            tf.printList()
        done()
    except:
        exchandler('Failed to list blogs.')



def addBlog(name=None, usr=None, pwd=None, url=None, com=None):
    vmsg('Add and configure a blog')
    if not name:
        vmsg('Make the name short and unique, you will type it often')
        name = uinput('Name of the blog: ').strip()
    if theConfig.getBlogList().count(name) != 0:
        raise BlognameAlreadyExists, '%s IS configured' % (name,)
    if name.count(' ') > 0:
        raise MalformedIdentifier, 'spaces are not allowed for blog names'
    if not usr:
        vmsg('The login name of your blog account')
        usr = uinput('Username: ').strip()
    if not pwd:
        # TODO: A password should be typed invisible
        vmsg('The password of your blog account')
        pwd = uinput('Password: ').strip()
    if not url:
        vmsg('The URL of your blog')
        url = uinput('Blog URL: ').strip()
    if not com:
        vmsg('A comment helping you to identify this blog')
        com = uinput('Comment: ').strip()
    try:
        vmsg('Adding %s...', name)
        vmsg('Autoconfiguring %s...', name)
        bc = blog.configureBlog(name, url, usr, pwd, com)
        vmsg('Saving %s to configuration...', name)
        done()
        theConfig.saveBlogConfig(bc)
    except:
        exchandler('failed to add %s.', name)



def removeBlog(name):
    try:
        if not name:
            name = uinput('Name of the blog to remove: ').strip()
        vmsg('Removing %s from the configuration...', name)
        theConfig.deleteBlogConfig(name)
        done()
    except:
        exchandler('failed to remove blog %s.', name)



def visitBlog(name):
    try:
        if not name:
            name = uinput('Name of the blog to visit: ').strip()
        vmsg('Visiting %s with the web browser.', name)
        bc = theConfig.getBlogConfig(name)
        bc.visit()
        done()
    except:
        exchandler('failed to visit blog %s.', name)



def blogInformation(name):
    try:
        if not name:
            name = uinput('Name of the blog for details: ').strip()
        vmsg('Blog information for %s', name)
        bc = theConfig.getBlogConfig(name)
        tf = wpshbase.TupleFormatter()
        tf.add('Name', bc.name)
        tf.add('URL', bc.url)
        tf.add('Blog-ID', bc.id)
        tf.add('XMLRPC-URL', bc.xmlurl)
        tf.add('Categories', ', '.join(bc.cats.keys()))
        tf.add('Admin-Flag', bc.admin)
        tf.add('Comment', bc.comment)
        tf.printList()
        done()
    except:
        exchandler('failed to get blog information for %s.', name)



def revalidateBlog(name):
    try:
        if not name:
            name = uinput('Name of the blog to revalidate: ').strip()
        vmsg('Revalidating %s...', name)
        bc = theConfig.getBlogConfig(name)
        if not bc.revalidate():
            raise RuntimeError, 'Failed to validate'
        vmsg('Saving the revalidated configuration...')
        theConfig.saveBlogConfig(bc)
        done()
    except:
        exchandler('failed to revalidate %s.', name)



def blogReadCategories(name):
    try:
        if not name:
            name = uinput('Name of the blog to read categories: ').strip()
        vmsg('Reading categories for %s', name)
        bc = theConfig.getBlogConfig(name)
        bc.readCategories()
        theConfig.saveBlogConfig(bc)
        done()
    except:
        exchandler('failed to read categories for %s.', name)



def blogRename(oldname, newname):
    try:
        vmsg('Renaming configuration %s to %s', oldname, newname)
        if theConfig.getBlogList().count(newname) > 0:
            raise BlognameAlreadyExists, '%s already exists' % (newname,)
        theConfig.renameBlogConfig(oldname, newname)
        done()
    except:
        exchandler('failed to rename configuration')


def blogComment(name, comment):
    try:
        vmsg('Setting comment for %s', name)
        bc = theConfig.getBlogConfig(name)
        bc.comment = comment
        vmsg('Saving modified blog configuration')
        theConfig.saveBlogConfig(bc)
        done()
    except:
        exchandler('failed to change blog comment')


def interactiveMode():
    try:
        vmsg('Starting interactive mode')
        sh = wpshell.WPShell()
        sh.cmdloop()
    except:
        exchandler('an unhandled exception occured');



def categoryTree(name):
    try:
        if not name:
            name = uinput('Name of the blog: ').strip()
        vmsg('Getting category tree')
        bc = theConfig.getBlogConfig(name)
        ct = bc.getCategoryTree()
        ct.printTree()
        done()
    except:
        exchandler('failed to get category tree')



def addCategory(bc, parentname = None):
    try:
        vmsg('Adding category to blog %s', bc.name)
        c = wt.Category()
        c.categoryName = uinput('Category Name: ').strip()
        if bc.cats.has_key(c.categoryName):
            raise CategoryAlreadyExists, '%s already exists' % (c.categoryName,)
        c.description = uinput('Description: ').strip()
        if parentname:
            vmsg('Retrieving ID for parent %s', parentname)
            if not bc.cats.has_key(parentname):
                raise CategoryDoesNotExist, 'not category %s' % (parentname,)
            pc = bc.cats[parentname]
            c.parentId = pc.categoryId
        vmsg('Creating new category')
        bc.newCategory(c)
        theConfig.saveBlogConfig(bc)
        attmsg("The category isn't configured. Do a 'config categories' soon")
        vmsg("To get fully functionality, do a 'config categories' soon")
        done()
    except:
        exchandler('failed to add category')



def removeCategory(bc, name):
    try:
        vmsg('Removing category %s', name)
        if not bc.cats.has_key(name):
            raise CategoryDoesNotExist, 'no category %s' % (name,)
        c = bc.cats[name]
        bc.removeCategory(c)
        theConfig.saveBlogConfig(bc)
        done()
    except:
        exchandler('failed to remove category')


def visitCategory(bc, name):
    try:
        vmsg('Visiting category %s', name);
        if not bc.cats.has_key(name):
            raise CategoryDoesNotExist, 'no category %s' % (parentname,)
        cat = bc.cats[name]
        cat.visit()
        done()
    except:
        exchandler('failed to visit category')



def blogPostTemplate(pattern):
    try:
        vmsg('Creating post templates for pattern "%s"', pattern)
        p = re.escape(pattern).replace(r'\*', '.*')
        flag = False
        for blogname in theConfig.getBlogList():
            if re.search(p, blogname):
                fname = blogname + '-Post.txt'
                vmsg('Creating %s for %s', fname, blogname)
                bc = theConfig.getBlogConfig(blogname)
                f = codecs.open(fname, 'wt', wpshbase.LOCALENC)
                f.write(bc.createPostTemplate())
                f.close()
                flag = True
        if not flag:
            attmsg('None of the configured blogs matches "%s"', pattern)
        done()
    except:
        exchandler('failed to create post templates')


# For future functionality

def blogPageTemplate(pattern):
    try:
        vmsg('Creating page templates for pattern "%s"', pattern)
        p = re.escape(pattern).replace(r'\*', '.*')
        flag = False
        for blogname in theConfig.getBlogList():
            if re.search(p, blogname):
                fname = blogname + '-Page.txt'
                vmsg('Creating %s for %s', fname, blogname)
                bc = theConfig.getBlogConfig(blogname)
                f = codecs.open(fname, 'wt', wpshbase.LOCALENC)
                f.write(bc.createPageTemplate())
                f.close()
                flag = True
        if not flag:
            attmsg('None of the configured blogs matches "%s"', pattern)
        done()
    except:
        exchandler('failed to create post templates')



def postFile(filename):
    try:
        inputParser = blog.InputParser()
        if filename == '-':
            vmsg('Posting from stdin')
            inputParser.readFileObject(sys.stdin)
            filename = '<stdin>'
        else:
            vmsg('Posting %s', filename)
            inputParser.readFile(filename)
        for metaPart in inputParser.getMetaParts():
            bc = metaPart.getBlogConfig()
            vmsg('Preparing %s for %s', filename, bc.name)
            instance = metaPart.getTypeInstance()
            instance.feed(metaPart, inputParser)
            id = bc.publishContent(instance, vmsg)
            vmsg('The ID of the created content is %d', id)
        done()
    except:
        exchandler('failed to publish file')
        

    
########################################################################
#
# $Log: commands.py,v $
# Revision 1.3  2009/09/20 02:31:37  elias
# Milestone: Can post with uploads
#
# Revision 1.2  2009/09/18 22:08:10  elias
# Intermediate state.
# Complete local blog administration.
#
# Revision 1.1  2009/09/17 15:33:25  elias
# Initial revision
#
########################################################################
#
# CODE IS PROSA, NOT POETRY!
#
########################################################################
