#!/usr/bin/env python
# -*- coding: utf-8 -*-
########################################################################
#
# wpsh/wpshbase.py
# $Id: wpshbase.py,v 1.3 2009/09/20 20:09:07 elias Alpha1 $
#
# The WordPress Shell
# Copyright (c) 2009 Elias Schwerdtfeger, http://www.tamagothi.de/ 
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
#
########################################################################

"""\
wpshbase.py

Base classes and utilities.

Part of this module are some hacks around future python compatibility
issues.

CfgParser is the configuration parser class
XmlRpc is the XMLRPC server proxy class
HtmlParser is the HTML parser class
CfgPath is the path of the configuration file

Variables
DEBUG - A debugging flag
ID - Version identifier
VERSION - Version number
LICENSE - A license announcement
WARRANTY - A (no-)warranty announcement
DONATE - A short text how to support me
LOCALENC - The encoding used locally

"""


import locale

import sys
import os.path


# Set Debug to False in stable release versions!
DEBUG = False

ID = '$Id: wpshbase.py,v 1.3 2009/09/20 20:09:07 elias Alpha1 $'

VERSION = '0.01alpha'

LOCALENC = locale.getpreferredencoding()

LICENSE = """
wpcmd -- The command line driven WordPress shell.
Copyright (c) 2009 Elias Schwerdtfeger, http://www.tamagothi.de/

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the 

    Free Software Foundation, Inc., 
    59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
"""

WARRANTY = """\
BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WAR‐
RANTY FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW.
EXCEPT WHEN OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS
AND/OR OTHER PARTIES PROVIDE THE PROGRAM ‘‘AS IS’’ WITHOUT WARRANTY
OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIM‐
ITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE.  THE ENTIRE RISK AS TO THE QUALITY AND PER‐
FORMANCE OF THE PROGRAM IS WITH YOU.  SHOULD THE PROGRAM PROVE
DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR
OR CORRECTION.
"""

DONATE = """\
If you like this nerdy little program, which empowers you to do
many WordPress tasks on a command line and even to do scripted
blogging in an easy way, please support me and the further deve-
lopment of this program.

I am an homeless artist and live by begging. I just give away all
the things I do and create, and I hope that something will return
to me. I may sound naive, but I can't live another way.

You can make your little donation via PayPal on my german homepage.
Just visit http://www.tamagothi.de/ and look for the sidebar ele-
ment titled "Spende", which is the german word for "donation."

And if you like it, take all the other stuff available there... ;-)
"""



CfgParser = None
HtmlParser = None
XmlRpc = None 
XmlBinary = None
CfgPath = None



if sys.hexversion >= 0x30000000:
    import configparser
    CfgParser = configparser.ConfigParser
else:
    import ConfigParser
    CfgParser = ConfigParser.ConfigParser



if sys.hexversion >= 0x30000000:
    import xmlrpc.client
    XmlRpc = xmlrpc.client.ServerProxy
    XmlBinary = xmlrpc.client.Binary
else:
    import xmlrpclib
    XmlRpc = xmlrpclib.ServerProxy
    XmlBinary = xmlrpclib.Binary



if sys.hexversion >= 0x30000000:
    import html.parser
    HtmlParser = html.parser.HTMLParser
else:
    import HTMLParser
    HtmlParser = HTMLParser.HTMLParser



# TODO: A better path and filename for other platforms than Unices.
# (How does ~ expand on Windows? What's about the dotted file?)

CfgPath = os.path.expanduser(os.path.join('~', '.wpcmdrc'))



# Please note that the doc string is displayed to the user in error messages.
# That's the reason for my so many exception classes, it makes printing
# errors much easier to me.
#
# If you hack in this code and there is a need for additional exceptions,
# give good doc-strings, roughly addressed to simple users who don't 
# understand nothing of all this hacking stuff and just wanna blog something
# on the commandline (or from an application).

class Error(RuntimeError):
    """Error"""
    pass



class InternalError(Error):
    """Internal Error"""
    pass



class DoubleKey(InternalError):
    """A unique name is specified twice"""
    pass



class Base:
    """\
    A generic base class.

    There is no need for __init__ in this class. It encapsulates just
    some useful functions for derived classes.

    PLUGIN MECHANISM

    Here is a simple plugin mechanism implemented.

    Every Plugin is realized via a callable object. It gets parameters
    as a tupel or list and returns the changed parameters as tupel or
    list. This is a simple mechanism, which allows for customization
    without hacking in the code of the core. 

    The used plugin keys are listed in the docstrings of the classes.
    """

    __plugins = {}

    @classmethod
    def addPlugin(cls, key, func):
        """\
        Add a plugin.

        Every plugin function gets a list and returns a list.

        key - Identifier
        func - Callable object to call
        
        Returns True on success
        """
        assert callable(func)
        if cls.__plugins.has_key(key):
            if cls.__plugins[key].count(func) == 0:
                cls.__plugins[key].append(func)
                return True
        else:
            cls.__plugins[key] = [func]
            return True
        return False

    @classmethod
    def removePlugin(cls, key, func):
        """\
        Remove a plugin for a key.
        
        key - Identifier
        func - Callable object to remove

        Returns True on success
        """
        assert callable(func)
        if not cls.__plugins.has_key(key):
            return False
        cls.__plugins[key] = [i for i in cls.__plugins[key] if i != func]
        return True

    @classmethod
    def removeAllPlugins(cls, key):
        """\
        Remove all plugins for a key.

        key - Identifier

        Returns always True
        """
        if cls.__plugins.has_key(key):
            del cls.__plugins[key]
        return True

    @classmethod
    def doPlugins(cls, key, args):
        """\
        Execute all plugins for an identifier.

        txt - Identifier
        args - The arguments as tupel or list

        Returns the filtered arguments as tupel or list

        NOTICE:
        The type of the argument list is enforced by assert statements,
        which is a little defensive coded, but I don't want to get 
        strange errors from deep core code malfunctioning. If you need
        full performance, you should -O optimize them away. But the
        XMLRPC and network traffic is the real bottleneck here, so the
        asserts should not be harmful.
        """
        assert isinstance(args, list) or isinstance(args, tuple)
        if cls.__plugins.has_key(key):
            for func in cls.__plugins[key]:
                args = func(args)
                assert isinstance(args, list) or isinstance(args, tuple)
        return args

    @classmethod
    def doPluginsArg(cls, key, *args):
        """\
        Simpler interface to the executePlugin method.

        This interface may be useful in cases where the arguments are
        not represented as a list internally.

        txt - Identifier
        args - The arguments

        Returns the filtered arguments as tupel or list
        """
        return cls.doPlugins(key, args)



class TupleFormatter(Base):
    """\
    A helper class for printing Tuples of information.

    Instances of this class accumulates the tuples with the add() method,
    the sorted list is printed with the printList method. 
    """

    def __init__(self, pluginSlot=None):
        """\
        Constructor.
        
        pluginSlot - A Slotname for a plugin filtering the keys
        """
        self.__a = {}
        self.__pluginslot = pluginSlot

    def add(self, key, value):
        """\
        Add a tupel to the printer instance.

        key - The key
        value - The associated text or other stuff
        """
        key = unicode(key, errors='ignore')
        if self.__a.has_key(key):
            raise DoubleKey, 'double key "%s"' % (key,)
        self.__a[key] = value
        
    def printList(self, indent=0):
        """\
        Print the list.

        indent - Number of spaces for indentation (optional)
        """
        lst = []
        if self.__pluginslot:
            keys = self.doPlugins(self.__pluginslot, self.__a.keys())
        else:
            keys = self.__a.keys()
        maxlen = max([len(i) for i in keys])
        firstfmt = u"%%-%ds" % (maxlen,)
        keys.sort(lambda x, y: cmp(x.lower(), y.lower()))
        for i in keys:
            if indent > 0:
                print ' ' * indent, firstfmt % (i,), '-', self.__a[i]
            else:
                print firstfmt % (i,), '-', self.__a[i]




########################################################################
#
# $Log: wpshbase.py,v $
# Revision 1.3  2009/09/20 20:09:07  elias
# Source cleaned up for alpha release.
# DEBUG set to False.
#
# Revision 1.2  2009/09/20 02:22:34  elias
# Milestone: Post with binary Uploads
#
# Revision 1.1  2009/09/17 11:45:07  elias
# Initial revision
#
########################################################################
#
# CODE IS PROSA, NOT POETRY!
#
########################################################################

