#!/usr/bin/env python
# -*- coding: utf-8 -*-
########################################################################
#
# wpsh/blog.py
# $Id: blog.py,v 1.6 2009/09/20 20:05:58 elias Alpha1 $
#
# The WordPress Shell
# Copyright (c) 2009 Elias Schwerdtfeger, http://www.tamagothi.de/ 
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
#
########################################################################

"""\
blog.py

The interface for blog configuration.
"""


import re
import os
import os.path
import webbrowser
import urllib
import codecs
import cPickle
import xml.dom.minidom
import mimetypes

import wpshbase as wp
import wptypes as wt



ID='$Id: blog.py,v 1.6 2009/09/20 20:05:58 elias Alpha1 $'



# Now the password isn't stored as cleartext anymore, but rot13 isn't
# the right way to spell security. But the config file is readable for
# its owner only, so it isn't *that* harmful.
#
# But for a later version here should be coded a better encryption. It
# will still be weak. This source is available for free, and the default
# path of the configuration file is world-known. If someone wants to
# attack it, he will try to exploit a browser or mailclient security
# flaw and get access to the file -- and to a lot of passwords too.
#
# It would be much better to encode the whole configuration file with
# an encryption hard to break and a key not easy to guess.
#
# It is a general problem.
# Any suggestions?

PASSWORD_ENCODING='rot13'
PasswordCodec = codecs.lookup(PASSWORD_ENCODING)



class NoSuchBlog(wp.Error):
    """Blog does not exist or isn't configured"""
    pass



class InvalidConfiguration(wp.Error):
    """Blog isn't valid"""
    pass



class BlogError(wp.Error):
    """General blog communication error"""
    pass



class NoRSD(BlogError):
    """Failed to read RSD-URL from the blog's HTML"""
    pass



class ValidateError(BlogError):
    """Failed to validate the settings"""
    pass



class XMLRPCError(BlogError):
    """General error in XMLRPC"""
    pass



class InputError(wp.Error):
    """General input error"""
    pass



class DoubleTitle(InputError):
    """Title specified twice"""
    pass



class MissingTitle(InputError):
    """No title given"""
    pass



class DoubleType(InputError):
    """Type specified twice"""
    pass



class IllegalType(InputError):
    """Illegal type specification"""
    pass



class NoBlogContext(InputError):
    """Missing blog context"""
    pass



class MissingMeta(InputError):
    """Missing meta information"""
    pass



class BlogConfig(wp.Base):
    """\
    Configuration for a blog.

    COMPONENTS:
    id - The blog ID
    name - The internal name of the blog
    url - The URL of the blog
    user - The username for logins
    pwd - The password for logins
    xmlurl - The XMLRPC entry URL
    cats - The list of configured categories
    admin - Is the user an administrator
    comment - A internal comment

    If you want to add a new blog, create an instance with the default
    constructor and fill the fields. Save it with the saveBlogConfig
    method of a Config instance.
    """

    def __init__(self, blogname=None, cfgobj=None):
        """\
        Constructor.

        Normally, the arguments are given by the framework. If you want
        to add a new blog, set the components manually and save it in 
        a Config file.

        The configureBlog() function is the easy way.

        PLUGIN SLOTS:
        EncryptLogin - To encrypt a username/password
        DecryptLogin - To decrypt a username/password
        """
        self.clear()
        if blogname and cfgobj:
            try:
                self.name = blogname
                self.url = cfgobj.get(blogname, 'BlogURL')
                self.id = cfgobj.getint(blogname, 'BlogID')
                u = cfgobj.get(blogname, 'Username')
                p = cfgobj.get(blogname, 'Password')
                self.user, self.pwd = self.doPluginsArg('DecryptLogin', u, p)
                self.xmlurl = cfgobj.get(blogname, 'XMLRPCURL')
                c = cfgobj.get(blogname, 'Categories')
                self.cats = cPickle.loads(c)
                self.admin = cfgobj.getboolean(blogname, 'CanAdmin')
                self.comment = cfgobj.get(blogname, 'Comment')
                self.__ok = True
            except:
                self.__ok = False

    def clear(self):
        """\
        Clear the configuration.
        """
        self.__ok = False
        self.__xmlproxy = None
        self.__xmlproxyok = False
        self.__cattree = None
        self.name = ''
        self.url = ''
        self.id = 0
        self.user = ''
        self.pwd = ''
        self.xmlurl = ''
        self.cats = {}
        self.admin = False
        self.comment = ''

    @staticmethod
    def encryptLogin(lst):
        """\
        Plugin function to encrypt the login information for storage.
        """
        return [PasswordCodec.encode(i)[0] for i in lst]
        
    @staticmethod
    def decryptLogin(lst):
        """\
        Plugin function to decrypt login information from storage.
        """
        return [PasswordCodec.decode(i)[0] for i in lst]

    def writeToConfigFile(self, cfgobj):
        """\
        Write the configuration to a config file object.

        This is a more internal function, used by various methods of
        the Config class.

        PLUGINS SLOTS
        EncryptLogin - Encrypt username and password for storage.
        """
        if not cfgobj.has_section(self.name):
            cfgobj.add_section(self.name)
        u, p = self.doPluginsArg('EncryptLogin', self.user, self.pwd)
        c = cPickle.dumps(self.cats)
        cfgobj.set(self.name, 'BlogURL', self.url)
        cfgobj.set(self.name, 'BlogID', self.id)
        cfgobj.set(self.name, 'Username', u)
        cfgobj.set(self.name, 'Password', p)
        cfgobj.set(self.name, 'XMLRPCURL', self.xmlurl)
        cfgobj.set(self.name, 'Categories', c)
        cfgobj.set(self.name, 'CanAdmin', self.admin)
        cfgobj.set(self.name, 'Comment', self.comment)

    def isValid(self):
        """\
        Is this blog valid and configured?
        """
        if not self.__ok:
            return False
        elif not self.xmlurl:
            return False
        elif not self.user:
            return False
        elif not self.pwd:
            return False
        else:
            return True

    def getServerProxy(self):
        """\
        Get a XMLRPC server proxy.

        Returns the server proxy
        """
        # I cannot use a simple test to self.__xmlproxy and have to
        # keep an additional instance variable. Sometimes the python magic
        # is rather black magic...
        if not self.__xmlproxyok:
            if not self.isValid():
                raise InvalidConfiguration, 'BlogConfig is invalid'
            self.__xmlproxy = wp.XmlRpc(self.xmlurl)
            self.__xmlproxyok = True
        return self.__xmlproxy

    def readCategories(self):
        """\
        Read the list of categories from the blog.

        On failure, the configured blog is marked as invalid.
        """
        self.cats = {}
        x = self.getServerProxy()
        for c in x.metaWeblog.getCategories(self.id, self.user, self.pwd):
            cobj = wt.Category(c)
            self.cats[cobj.categoryName] = cobj
        self.__cattree = None

    def revalidate(self):
        """\
        Validate the blog.

        This is done internally by checking user access to the blog and
        reading the list of categories. If this succeeds, the blog is valid.

        NOTICE:
        It will only word with new WordPress versions and uses some fresh
        stuff from the API. 

        Returns True on Success
        """
        self.__ok = True
        x = self.getServerProxy()
        flag = False
        for b in x.wp.getUsersBlogs(self.user, self.pwd):
            if b['url'].startswith(self.url):
                self.admin = bool(b['isAdmin'])
                self.url = b['url']
                self.id = int(b['blogid'])
                flag = True
                break
        if flag:
            try:
                self.readCategories()
            except xmlrpclib.Fault:
                self.__ok = False
            return self.__ok
        else:
            self.__ok = False
            return False

    def __str__(self):
        """\
        Convert it to a string.

        This may be useful for outputting in an interactive environment.
        
        If the blog isn't configured, the text "Unconfigured blog" is
        generated, in any other cases the configuration will be returned,
        but without the password.
        """
        if self.isValid():
            return u'\n'.join(self.getConfigLines())
        else:
            return 'Unconfigured blog'

    def visit(self):
        """\
        Visit the blog with your web browser.
        """
        if not self.isValid():
            raise InvalidConfiguration, 'BlogConfig is invalid'
        webbrowser.open(self.url)

    def getCategoryTree(self):
        """\
        Get the category tree for this blog.

        Returns a CategoryTree instance.
        """
        if not self.isValid():
            raise InvalidConfiguration, 'BlogConfig is invalid'
        if not self.__cattree:
            self.__cattree = wt.CategoryTree(self.cats)
        return self.__cattree

    def newCategory(self, cat):
        """\
        Create a new category.

        cat - A Category instance
        """
        if not self.isValid():
            raise InvalidConfiguration, 'BlogConfig is invalid'
        hsh = { 'name': cat.categoryName, 
                'description': cat.description }
        if cat.parentId:
            hsh['parent_id'] = cat.parentId
        x = self.getServerProxy()
        id = x.wp.newCategory(self.id, self.user, self.pwd, hsh)
        if not id:
            raise XMLRPCError, 'strange internal error'
        cat.categoryId = id
        self.cats[cat.categoryName] = cat
        self.__cattree = None

    def removeCategory(self, cat):
        """\
        Remove a category.

        cat - A Category instance
        """
        if not self.isValid():
            raise InvalidConfiguration, 'BlogConfig is invalid'
        x = self.getServerProxy()
        x.wp.deleteCategory(self.id, self.user, self.pwd, cat.categoryId)
        del self.cats[cat.categoryName]
        self.__cattree = None

    def __templateHelper(self, a):
        """\
        A helper method for creating templates.
        """
        a.insert(0, 'blog: %s' % (self.name,))
        a = ['!!' + i for i in a]
        a.insert(0, '# Template for %s' % (self.name))
        a.insert(0, '# -*- coding: %s -*-' % (wp.LOCALENC.lower(),))
        a.append('!!title: ')
        return u'\n'.join(a) + '\n\n'

    def createPostTemplate(self):
        """\
        Create a post template for an instance.

        Returns string with the template
        """
        a = ['type: post']
        # Do some funny formatting with the categories to make the
        # lines fit in 74 column displays and so easier to edit...
        offset = len('!!categories: ')
        catlist = self.cats.keys()
        catlist.sort()
        worker = []
        colcnt = 0
        for catname in catlist:
            catlen = len(catname) + 2
            if colcnt + catlen + offset > 74:
                a.append('categories: %s' % (', '.join(worker),))
                worker = []
                colcnt = 0
            else:
                worker.append(catname)
                colcnt += catlen
        if len(worker) > 0:
            a.append('categories: %s' % (', '.join(worker),))
        a.append('tags: ')
        return self.__templateHelper(a)

    def createPageTemplate(self):
        """\
        Create a page template for an instance.
        
        Returns string with the template
        """
        a = ['type: page']
        return self.__templateHelper(a)

    def publishContent(self, instance, msgcb=None):
        """Publish a content instance"""
        x = self.getServerProxy()
        uploads = instance.getUploadArray()
        if len(uploads) > 0:
            for filename in uploads:
                if msgcb:
                    msgcb('Uploading "%s"...', filename)
                s = { }
                fp = open(filename, 'rb')
                s['name'] = os.path.basename(filename)
                s['type'] = mimetypes.guess_type(filename)[0]
                s['bits'] = wp.XmlBinary(fp.read())
                fp.close()
                h = x.wp.uploadFile(self.id, self.user, self.pwd, s)
                instance.registerUploadedPath(filename, h['url'])
            instance.replaceUploadUris()
            if msgcb:
                msgcb('Sending text...')
        s = instance.getXMLRPCHash()
        id = x.metaWeblog.newPost(self.id, self.user, self.pwd, s)
        return int(id)



class Config(wp.Base):
    """\
    Interface to the configuration of all blogs.

    In normal applications, there is only one instance of this class.
    The blogs are configured and accessed here.
    """

    def __init__(self):
        """\
        Constructor.

        PLUGIN SLOTS:
        Config.CfgPath - to change the path of the configuration file
        Config.Blognames - to filter the blognames
        """
        self.__c = wp.CfgParser()
        self.__b = {}
        (self.__cfgpath,) = self.doPluginsArg('Config.CfgPath', wp.CfgPath)
        if os.path.exists(self.__cfgpath):
            self.__c.read(self.__cfgpath)
            sects = self.doPlugins('Config.Blognames', self.__c.sections())
            for sect in sects:
                self.__b[sect] = None

    def __save(self):
        """
        Save a new configuration to the config file.
        """
        oldumask = -1
        try:
            oldumask = os.umask(0177)
        except:
            pass
        fp = open(self.__cfgpath, 'wb')
        self.__c.write(fp)
        fp.close()
        if oldumask != -1:
            os.umask(oldumask)

    def getBlogList(self):
        """\
        Get a list of configured blogs.

        Returns the configured blogs as a list.

        PLUGIN SLOTS:
        Config.GetBlogList - to filter the blog list
        """
        l = self.__b.keys()
        return self.doPlugins('Config.GetBlogList', l)
                
    def getBlogConfig(self, name):
        """\
        Get the blog configuration for a specified name.

        Returns the Blog object.
        """
        if not self.__b.has_key(name):
            raise NoSuchBlog, 'No configuration for %s' % (name,)
        if not self.__b[name]:
            self.__b[name] = BlogConfig(name, self.__c)
        return self.__b[name]

    def saveBlogConfig(self, bc):
        """\
        Add a new or changed blog configuration.

        You have to fill the components manually.

        bc - The blog config object
        """
        if not bc.isValid():
            raise InvalidConfiguration, 'BlogConfig is invalid'
        self.__b[bc.name] = bc
        bc.writeToConfigFile(self.__c)
        self.__save()

    def renameBlogConfig(self, oldname, newname):
        """\
        Rename a blog configuration.

        oldname - the old name
        newname - the new name
        """
        bc = self.getBlogConfig(oldname)
        self.__c.remove_section(oldname)
        del self.__b[oldname]
        bc.name = newname
        self.__b[newname] = bc
        self.saveBlogConfig(bc)

    def deleteBlogConfig(self, name):
        """\
        Delete a blog configuration.

        name - The name of the blog
        """
        if not self.__b.has_key(name):
            raise NoSuchBlog, '%s is not configured' % (name,)
        del self.__b[name]
        self.__c.remove_section(name)
        self.__save()



theConfig = Config()



class RSDGetter(wp.HtmlParser):
    """\
    Get the RSD informations (EditURI) from the HTML source.

    A helper class.

    This works with a WordPress Blog, and isn't tested with any other
    system, but my script is written for WordPress, so don't expect it
    to work with other blog software.

    KNOWNBUG
    Does not work with non-ASCII characters in an URL currently.
    """

    def handle_starttag(self, tag, attr):
        """\
        Extract a link-tag with type application/rsd+xml.
        """
        if tag != 'link':
            return
        ah = {}
        for key, val in attr:
            ah[key] = val
        if not ah.has_key('type'):
            return
        if ah['type'].lower() == 'application/rsd+xml':
            if self.__found:
                raise RuntimeError, 'Double RSD spec'
            self.__found = ah['href']

    def getRSDFromURL(self, url):
        self.__found = None
        self.reset()
        # KNOWN BUG!
        #
        # Must use codec to ignore errors with charsets. Oh, how I hate
        # this sucking python! Thanks Bog for the strict ASCII in
        # most URLs, but this will be problematic with modern-style URLs
        # containing funny letters, so I have to fix it later. TODO.
        #
        # (An almost always working way of fixing is to assume that the 
        # charset is UTF-8, which is a WordPress default changed seldomly.
        # But I'm still thinking on better ways, and my alpha release can
        # be buggy.)
        f = codecs.EncodedFile(urllib.urlopen(url), 'ascii', 'ascii', 'ignore')
        self.feed(f.read())
        f.close()
        return self.__found



def configureBlog(name, url, user, pwrd, comment=''):
    """\
    Create a new BlogConfig object for a blog.
    """
    bc = BlogConfig()
    # Enforce http:// specification if not given.
    # Many folks do believe www. is a protocol...
    # But if someone wants https:, this must be typed manually.
    if not url.startswith('http'):
        url = 'http://' + url
    # Get the XMLRPC entry URL
    rsdget = RSDGetter()
    rsdurl = rsdget.getRSDFromURL(url)
    if not rsdurl:
        raise NoRSD, 'Failed to read RSD from %s' % (url)
    # Parse the relevant stuff from the RSD DOMtree (very quick and dirty)
    f = urllib.urlopen(rsdurl)
    doc = xml.dom.minidom.parse(f)
    f.close()
    blogid = None
    for a in doc.getElementsByTagName('api'):
        if a.getAttribute('name') == 'WordPress':
            blogid = int(a.getAttribute('blogID'))
            xmlrpc = a.getAttribute('apiLink')
            break
    if not blogid:
        raise NoRSD, 'Is this a WordPress blog?'
    # Write it to the clean configuration
    bc.name = name
    bc.url = url
    bc.user = user
    bc.pwd = pwrd
    bc.id = blogid
    bc.xmlurl = xmlrpc
    bc.comment = comment
    # Validate this configuration
    if not bc.revalidate():
        raise ValidateError, 'Could not validate the settings'
    # And return it. That's simple, isn't it...
    return bc



# You may want to change this...
# Just remove these plugins and do your own work in your app.
#
# TODO: rot13 isn't the proper way to spell security!

BlogConfig.addPlugin('DecryptLogin', BlogConfig.decryptLogin)
BlogConfig.addPlugin('EncryptLogin', BlogConfig.encryptLogin)



class InputMetaPart(wp.Base):
    """\
    A meta part of an input file.

    Every blog has a meta part, which is an representation of all 
    those !! lines in a file.

    COMPONENTS:
    meta - The raw meta data
    """

    def __init__(self, blogname):
        """\
        Constructor.

        blogname - The name of the blog
        """
        self.__blogname = blogname
        self.__meta = {}
        self.__bc = theConfig.getBlogConfig(blogname)

    def addMeta(self, metakey, metadata):
        """\
        Add a line of meta information to the part.

        metakey - The keyword
        metadata - The associated data
        """
        metadata = metadata.strip()
        # Do not add empty strings
        if len(metadata) == 0:
            return
        metakey = metakey.lower()
        if not self.__meta.has_key(metakey):
            self.__meta[metakey] = [metadata]
        else:
            self.__meta[metakey].append(metadata)

    def getRawMeta(self):
        """\
        Get the raw meta data.

        Returns a hash to arrays.
        """
        return self.__meta

    def getMetaArray(self, metakey):
        """\
        Get the meta information as array.

        metakey - The keyword

        Returns an array which may be empty on undefined keys
        """
        if self.__meta.has_key(metakey):
            return self.__meta[metakey]
        else:
            return []
        
    def getMetaCommaSeparated(self, metakey):
        """\
        Get the meta information comma separated.
       
        metakey - The keyword
        """
        return ', '.join(self.getMetaArray(metakey))
            
    def getTitle(self):
        """\
        Get the title for the Blog.
        """
        if not self.__meta.has_key('title'):
            raise MissingTitle, 'no title for to %s' % (self.blogname,)
        a = self.__meta['title']
        if len(a) > 1:
            raise DoubleTitle, 'title not unique to %s' % (self.blogname,)
        return a[0]

    def getType(self):
        """\
        Get the type of this meta part.

        It should be page or post, perhaps something more in future.

        If no type is given, it will be a post by default. This is the
        most frequently used type.
        """
        if not self.__meta.has_key('type'):
            t = 'post'
        else:
            a = self.__meta['type']
            if len(a) > 1:
                raise DoubleType, 'type not unique to %s' % (self.blogname,)
            # Make lowercase for error tolerance -- users are strange
            t = a[0].lower() 
        if t not in ('post', 'page'):
            raise IllegalType, 'unsupported type %s' % (t,)
        return t

    def getTypeInstance(self):
        """\
        Get the matching instance for the chose type of the meta.
        """
        t = self.getType()
        if t == 'post':
            return wt.Post()
        elif t == 'page':
            return wt.Page()
        else:
            raise NotImplementedError, 'unsupported type %s' % (t, )

    def getBlogConfig(self):
        """Get the blog config for this meta part"""
        return self.__bc

    def getBlogName(self):
        """Get the blog name for this meta part"""
        return self.__blogname


class InputParser(wp.Base):
    """\
    The input parser.

    This is the workhorse for textfile posting. It is ugly code, but it
    must handle human input which may be irregular. The "uglyness" of the
    code reflects the computers degree of stupidity.
    """

    # A comment line
    com_re = re.compile(r'\#')
    # A wpcmd meta line
    met_re = re.compile(r'\!\!([a-z]*)\:\s*(.*)')
    # A encoding specification
    enc_re = re.compile(r'\-\*\-\s*encoding\:\s*([0-9a-z\-]+)\s*\-\*\-', re.I)

    def __init__(self):
        """Constructor."""
        self.__text = ''
        self.__parag = []
        self.__mparts = {}
        self.__aktmp = None

    def __addMeta(self, matchobj):
        """Add a matched meta."""
        k = matchobj.group(1).lower()
        ln = matchobj.group(2)
        # Define a new blog context
        if k == 'blog':
            if self.__mparts.has_key(ln):
                self.__aktmp = self.__mparts[ln]
            else:
                self.__aktmp = InputMetaPart(ln)
                self.__mparts[ln] = self.__aktmp
        # Or add lines to the blog context
        else:
            if not self.__aktmp:
                raise NoBlogContext, 'no blog specified for %s', (ln,)
            self.__aktmp.addMeta(k, ln)

    def readFileObject(self, fileobj):
        """\
        Read a file given by a file object.

        fileobj - The file object
        """
        enc = wp.LOCALENC
        la = fileobj.readlines()
        # Look for encoding spec before processing
        for ln in la:
            mo = self.enc_re.search(ln)
            if mo:
                enc = mo.group(1)
                break
        # Decode a given encoding 
        if enc: 
            la = [ln.decode(enc) for ln in la]
        # Remove comment lines and strip lines, don't need these
        # trailing newlines
        la = [ln.strip() for ln in la if not self.com_re.match(ln)]
        # Process the lines and separate text from meta
        txtl = []
        for ln in la:
            mo = self.met_re.match(ln)
            if mo:
                self.__addMeta(mo)
            else:
                txtl.append(ln)
        # Now we have textlines and meta parts separated, and the
        # meta parts are fine, so we should be ready to process 
        # the text. All stuff in a paragraph should become a long
        # line, this is the way wordpress can handle it automatically.
        parag = []
        work = []
        for ln in txtl:
            if len(ln) == 0:
                # This is an empty line, so we put the work as paragraph
                # if there is any.
                if len(work) > 0:
                    parag.append(u' '.join(work))
                    work = []
            else:
                # This is a non-empty line, append it to the worker array.
                work.append(ln)
        # There may be more work to do after the loop, don't want to lose
        # the last paragraph
        if len(work) > 0:
            parag.append(' '.join(work))
        self.__parag = parag
        # Now we have an array of paragraphs we can simply join together
        self.__text = u'\n\n'.join(parag)

    def readFile(self, filename):
        """\
        Read a file given by name.

        filename - The filename
        """
        f = open(filename, 'r')
        self.readFileObject(f)
        f.close()

    def getMetaParts(self):
        """Get the meta parts"""
        if len(self.__mparts) == 0:
            raise MissingMeta, 'No meta information in the input'
        return self.__mparts.values()

    def getText(self):
        """Get the text"""
        return self.__text

    def getParagraphs(self):
        """Get the paragraphs of the text."""
        return self.__parag



########################################################################
#
# $Log: blog.py,v $
# Revision 1.6  2009/09/20 20:05:58  elias
# Source cleaned up for first alpha release.
#
# Revision 1.5  2009/09/20 04:08:33  elias
# Changed identification of the correct blog in BlogConfig.validate().
# The blogid in WordPress' RSD is wrong! Use with a WPMU, and disaster.
# I check for the URL in the list of wp.getUserBlogs() now, which isn't
# backward compatible.
#
# Revision 1.4  2009/09/20 02:22:05  elias
# publishContent publishes Posts with uploads.
#
# Revision 1.3  2009/09/20 00:38:08  elias
# Milestone: can post but not upload.
#
# Revision 1.2  2009/09/18 22:07:05  elias
# Intermediate state.
# Local blog administration completed.
#
# Revision 1.1  2009/09/16 20:31:45  elias
# Initial revision
#
########################################################################
#
# CODE IS PROSA, NOT POETRY!
#
########################################################################
